/*
 	File:		DSP3210.h
 
 	Contains:	Interface to the Macintosh DSP3210 API
 
 	Version:	Technology:	
 				Package:	Universal Interfaces  DDK v23c5.  Monday, May 15, 1995.
 
 	Copyright:	 1984-1995 by Apple Computer, Inc.
 				All rights reserved.
 
 	Bugs?:		If you find a problem with this file, use the Apple Bug Reporter
 				stack.  Include the file and version information (from above)
 				in the problem description and send to:
 					Internet:	apple.bugs@applelink.apple.com
 					AppleLink:	APPLE.BUGS
 
*/

#ifndef __DSP3210__
#define __DSP3210__


#ifdef __cplusplus
extern "C" {
#endif

#if STRUCTALIGNMENTSUPPORTED
#pragma options align=mac68k
#endif

#if PRAGMA_IMPORT_SUPPORTED
#pragma import on
#endif


#ifndef __DSP__
#include <DSP.h>
#endif
#define NewModule(Name, GPBFlags, ModuleFlags, EntryName)  \
	@lowmod(Name,GPBFlags,ModuleFlags,EntryName)
#define NewSection(Name, SectionFlags, SectionDataType, ModuleName)  \
	@lowseg(Name,SectionFlags,0,SectionDataType,ModuleName)
#define AppendSection(Name) 	\
	.rsect	"Name", TEXT
#define NewInputFIFOAndBufferSection(Name, BufferSize, SectionDataType, ModuleName)  \
	NewSection(Name,			\
	kdspBankB | kdspSaveOnContextSwitch | kdspDSPUseOnly | kdspInputBuffer | kdspFIFOSection, \
	SectionDataType,			\
	ModuleName)					\
	BufferSize * long 0
#define NewOutputFIFOAndBufferSection(Name, BufferSize, SectionDataType, ModuleName)  \
	NewSection(Name,(kdspBankB | kdspSaveOnContextSwitch | kdspDSPUseOnly | kdspOutputBuffer | kdspFIFOSection),SectionDataType,ModuleName) \
	BufferSize * long 0
#define NewInputFIFOAndScalableBufferSection(Name, BufferScale, SectionDataType, ModuleName)  \
	NewSection(Name,			\
	kdspBankB | kdspSaveOnContextSwitch | kdspDSPUseOnly | kdspInputBuffer | kdspFIFOSection | kdspScalableSection, \
	SectionDataType,			\
	ModuleName)					\
	BufferScale * long 0
#define NewOutputFIFOAndScalableBufferSection(Name, BufferScale, SectionDataType, ModuleName)  \
	NewSection(Name,			\
	kdspBankB | kdspSaveOnContextSwitch | kdspDSPUseOnly | kdspOutputBuffer | kdspFIFOSection | kdspScalableSection, \
	SectionDataType,			\
	ModuleName)					\
	BufferScale * long 0
#define NewInputAIAOSection(Name, AIAOSize, SectionDataType, ModuleName)  \
	NewSection(Name,			\
	kdspLoadSection | kdspBankB | kdspSaveOnContextSwitch | kdspDSPUseOnly|kdspInputBuffer | kdspStaticSection, \
	SectionDataType,			\
	ModuleName)					\
	AIAOSize * long 0
#define NewOutputPRBSection(Name, AIAOSize, SectionDataType, ModuleName)  \
	NewSection(Name,			\
	kdspClearSection | kdspSaveSection | kdspBankB | kdspSaveOnContextSwitch | kdspDSPUseOnly | kdspInputBuffer | kdspOutputBuffer | kdspStaticSection, \
	SectionDataType,			\
	ModuleName)					\
	AIAOSize * long 0
#define NewOutputCRBSection(Name, AIAOSize, SectionDataType, ModuleName)  \
	NewSection(Name,			\
	kdspSaveSection | kdspBankB | kdspSaveOnContextSwitch | kdspDSPUseOnly | kdspOutputBuffer | kdspStaticSection, \
	SectionDataType,			\
	ModuleName)					\
	AIAOSize * long 0
#define NewScalableInputAIAOSection(Name, AIAOScale, SectionDataType, ModuleName)  \
	NewSection(Name,			\
	kdspLoadSection | kdspBankB | kdspSaveOnContextSwitch | kdspDSPUseOnly | kdspInputBuffer | kdspStaticSection | kdspScalableSection, \
	SectionDataType,			\
	ModuleName)					\
	AIAOScale * long 0
#define NewTempScalableAIAOSection(Name, AIAOScale, SectionDataType, ModuleName)  \
	NewSection(Name,			\
	kdspLeaveSection | kdspBankB | kdspSaveOnContextSwitch | kdspDSPUseOnly | kdspScalableSection | kdspClearSection, \
	SectionDataType,			\
	ModuleName)					\
	AIAOScale * long 0
#define NewScalableOutputPRBSection(Name, AIAOScale, SectionDataType, ModuleName)  \
	NewSection(Name,			\
	kdspClearSection | kdspSaveSection | kdspBankB | kdspSaveOnContextSwitch | kdspDSPUseOnly | kdspInputBuffer | kdspOutputBuffer | kdspStaticSection | kdspScalableSection, \
	SectionDataType,			\
	ModuleName)					\
	AIAOScale * long 0
#define NewScalableOutputCRBSection(Name, AIAOScale, SectionDataType, ModuleName)  \
	NewSection(Name,			\
	kdspSaveSection | kdspBankB | kdspSaveOnContextSwitch | kdspDSPUseOnly| kdspOutputBuffer | kdspStaticSection | kdspScalableSection, \
	SectionDataType,			\
	ModuleName)					\
	AIAOScale * long 0
#define NewCachedProgramSection(Name, ModuleName)  \
	NewSection(Name,			\
	kdspLoadSection | kdspBankA | kdspDSPUseOnly | kdspNotIOBufferSection, \
	kdspNonData,				\
	ModuleName)
#define NewExternalProgramSection(Name, ModuleName)  \
	NewSection(Name,			\
	kdspLeaveSection|kdspNotIOBufferSection, \
	kdspNonData,				\
	ModuleName)
#define NewParameterSection(Name, SectionDataType, ModuleName)  \
	NewSection(Name,			\
	kdspExternal|kdspNotIOBufferSection, \
	SectionDataType,			\
	ModuleName)
#define NewTableSection(Name, SectionDataType, ModuleName)  \
	NewSection(Name,			\
	kdspLoadSection | kdspBankA | kdspDSPUseOnly|kdspNotIOBufferSection, \
	SectionDataType,			\
	ModuleName)
#define NewStateVariableSection(Name, SectionDataType, ModuleName)  \
	NewSection(Name,			\
	kdspLoadSection | kdspSaveSection | kdspBankB | kdspDSPUseOnly | kdspSaveOnContextSwitch | kdspNotIOBufferSection, \
	SectionDataType,			\
	ModuleName)
#define NewTempVariableSection(Name, SectionDataType, ModuleName)  \
	NewSection(Name,			\
	kdspLeaveSection | kdspBankB | kdspSaveOnContextSwitch | kdspDSPUseOnly | kdspNotIOBufferSection, \
	SectionDataType,			\
	ModuleName)
#define TMP r17
#define RV r18
#define MXPB r19
#define ERRT r20
#define SP r21
#define EVTP r22
#define kLongWordSize (4)
#define kStr31Size (32)
#define msVector (0)
#define msData (msVector + kLongWordSize)
#define DSPMessageSize (msData + 3*kLongWordSize)
#define kEVTPad (512)
#define Find(theSelector) EVTP + kEVTPad + theSelector*kLongWordSize
#define OSCall(theSelector) 	\
	RV = (long) Find(theSelector); \
	RV = (long) *RV;			\
	nop;						\
	call RV (RV);				\
	nop
#define kdspSelectPush 0x00000000
#define kdspSelectPop 0x00000001
#define kdspOptionSpecifyBuffer 0x00000002
#define kdspOptionSpecifyCount 0x00000004
#define kdspOptionNoCopyJustCount 0x00000008
#define BlockMove(theSrcPtr, theDestPtr, theCount)  \
	r1 = (long) theSrcPtr;		\
	r2 = (long) theDestPtr;		\
	r3 = (long) theCount;		\
	RV = (long) Find(kevtBlockMove); \
	RV = (long) *RV;			\
	r15 = (ushort24) 0x0004;	\
	call RV (RV);				\
	r16 = (ushort24) 0x0004
#define PcLabel(theSectionLabel) pc + ((theSectionLabel)-(.+8))
#define Pop(theRegister) 		\
	SP = (long) SP--;			\
	theRegister = (long) *SP;	\
	nop
#define Push(theRegister) *SP++ = (long) theRegister
#define CallSection(theSectionName)  \
	RV = (long) MXPB + sectn theSectionName; \
	RV = (long) *RV;			\
	nop;						\
	call RV (RV);				\
	nop
#define GetSectionAddress(theSectionPtr, theSectionName)  \
	theSectionPtr = (long) MXPB + sectn theSectionName; \
	theSectionPtr = (long) *theSectionPtr; \
	nop
#define GetSectionSize(theSectionSize, theSectionName)  \
	RV = (long) Find(kevtGetSectionSize); \
	RV = (long) *RV;			\
	r1 = (short) sectn theSectionName; \
	call RV (RV);				\
	nop;						\
	theSectionSize = (long) r2
#define GetSectionLabel(theSectionLabelPtr, theSectionLabel)  \
	theSectionLabelPtr = (long) MXPB + sectn theSectionLabel; \
	theSectionLabelPtr = (long) *theSectionLabelPtr; \
	nop;						\
	theSectionLabelPtr = (long) theSectionLabelPtr + offset theSectionLabel
#define PopSection(theSectionName)  \
	RV = (long) Find(kevtCacheSection); \
	RV = (long) *RV;			\
	r1 = (short) sectn theSectionName; \
	call RV (RV);				\
	r2 = (ushort24) kdspSelectPop
#define PushSection(theSectionName)  \
	RV = (long) Find(kevtCacheSection); \
	RV = (long) *RV;			\
	r1 = (short) sectn theSectionName; \
	call RV (RV);				\
	r2 = (ushort24) kdspSelectPush
#define SetSkipCount(theSkipCount)  \
	RV = (long) Find(kevtSetSkipCount); \
	RV = (long) *RV;			\
	r1 = (long) theSkipCount;	\
	call RV (RV);				\
	nop
#define GetNumRealTimeFrames(numFrames)  \
	numFrames = (long) Find(kevtNumRealTimeFrames); \
	numFrames = (long) *numFrames; \
	nop
#define SetTaskInactive() OSCall(kevtSetTaskInactive)
#define FIFOGetReadCount(theFIFOName)  \
	RV = (long) Find(kevtCopyFIFO); \
	RV = (long) *RV ;			\
	r1 = (short) sectn theFIFOName; \
	call RV (RV);				\
	r2 = (ushort24) (kdspOptionNoCopyJustCount | kdspSelectPush)
#define FIFOGetWriteCount(theFIFOName)  \
	RV = (long) Find(kevtCopyFIFO); \
	RV = (long) *RV;			\
	r1 = (short) sectn theFIFOName; \
	call RV (RV);				\
	r2 = (ushort24) (kdspOptionNoCopyJustCount | kdspSelectPop)
#define FIFORead(theSectionName)  \
	RV = (long) Find(kevtCopyFIFO); \
	RV = (long) *RV;			\
	r1 = (short) sectn theSectionName; \
	call RV (RV);				\
	r2 = (ushort24) (kdspSelectPush)
#define FIFOReadN(theFIFOName, theCount)  \
	r4 = (long) theCount;		\
	RV = (long) Find(kevtCopyFIFO); \
	RV = (long) *RV;			\
	r1 = (short) sectn theFIFOName; \
	call RV (RV);				\
	r2 = (ushort24) (kdspSelectPush | kdspOptionSpecifyCount)
#define FIFOReadNBuffer(theFIFOName, theCount, theBufferPtr)  \
	r3 = (long) theBufferPtr;	\
	r4 = (long) theCount;		\
	RV = (long) Find(kevtCopyFIFO); \
	RV = (long) *RV;			\
	r1 = (short) sectn theFIFOName; \
	call RV (RV);				\
	r2 = (ushort24) (kdspSelectPush | kdspOptionSpecifyCount | kdspOptionSpecifyBuffer)
#define FIFOWrite(theSectionName)  \
	RV = (long) Find(kevtCopyFIFO); \
	RV = (long) *RV;			\
	r1 = (short) sectn theSectionName; \
	call RV (RV);				\
	r2 = (ushort24) (kdspSelectPop)
#define FIFOWriteN(theFIFOName, theCount)  \
	r4 = (long) theCount;		\
	RV = (long) Find(kevtCopyFIFO); \
	RV = (long) *RV;			\
	r1 = (short) sectn theFIFOName; \
	call RV (RV);				\
	r2 = (ushort24) (kdspSelectPop | kdspOptionSpecifyCount)
#define FIFOWriteNBuffer(theFIFOName, theCount, theBufferPtr)  \
	r3 = (long) theBufferPtr;	\
	r4 = (long) theCount;		\
	RV = (long) Find(kevtCopyFIFO); \
	RV = (long) *RV;			\
	r1 = (short) sectn theFIFOName; \
	call RV (RV);				\
	r2 = (ushort24) (kdspSelectPop | kdspOptionSpecifyCount | kdspOptionSpecifyBuffer)
#define GPBElapsedCycles(theCycles)  \
	OSCall(kevtGPBElapsedCycles); \
	theCycles = (long) r2
#define GPBExpectedCycles(theCycles)  \
	OSCall(kevtGPBExpectedCycles); \
	theCycles = (long) r2
#define GPBSetUseActual() OSCall(kevtGPBSetUseActual)
#define GPBChangeMode() OSCall(kevtGPBChangeMode)
#define SendMessageToHost(theDSPMessagePtr)  \
	RV = (long) Find(kevtMessageToHost); \
	RV = (long) *RV;			\
	r1 = (long) theDSPMessagePtr; \
	call RV (RV);				\
	nop
#define SemaphoreClear(theSemaphorePtr, theMask, theOldSemaphoreValue)  \
	RV = (long) Find(kevtSemaphoreClear); \
	RV = (long) *RV;			\
	r1 = (long) theSemaphorePtr; \
	r2 = (long) theMask;		\
	call RV (RV);				\
	nop;						\
	theOldSemaphoreValue = (long) r3
#define SemaphoreSet(theSemaphorePtr, theMask, theOldSemaphoreValue)  \
	RV = (long) Find(kevtSemaphoreSet); \
	RV = (long) *RV;			\
	r1 = (long) theSemaphorePtr; \
	r2 = (long) theMask;		\
	call RV (RV);				\
	nop;						\
	theOldSemaphoreValue = (long) r3

#if PRAGMA_IMPORT_SUPPORTED
#pragma import off
#endif

#if STRUCTALIGNMENTSUPPORTED
#pragma options align=reset
#endif

#ifdef __cplusplus
}
#endif

#endif /* __DSP3210__ */
