/*
 	File:		ROMDefs.h
 
 	Contains:	NuBus card ROM Definitions
 
 	Version:	Technology:	
 				Package:	Universal Interfaces  DDK v23c5.  Monday, May 15, 1995.
 
 	Copyright:	 1984-1995 by Apple Computer, Inc.
 				All rights reserved.
 
 	Bugs?:		If you find a problem with this file, use the Apple Bug Reporter
 				stack.  Include the file and version information (from above)
 				in the problem description and send to:
 					Internet:	apple.bugs@applelink.apple.com
 					AppleLink:	APPLE.BUGS
 
*/

#ifndef __ROMDEFS__
#define __ROMDEFS__


#ifdef __cplusplus
extern "C" {
#endif

#if STRUCTALIGNMENTSUPPORTED
#pragma options align=mac68k
#endif

#if PRAGMA_IMPORT_SUPPORTED
#pragma import on
#endif


enum {
	appleFormat					= 1,							/*Format of Declaration Data (IEEE will assign real value)*/
	romRevision					= 1,							/*Revision of Declaration Data Format*/
	testPattern					= 1519594439,					/*FHeader long word test pattern*/
	sCodeRev					= 2,							/*Revision of code (For sExec)*/
	sCPU68000					= 1,							/*CPU type = 68000*/
	sCPU68020					= 2,							/*CPU type = 68020*/
	sCPU68030					= 3,							/*CPU type = 68030*/
	sCPU68040					= 4,							/*CPU type = 68040*/
	sMacOS68000					= 1,							/*Mac OS, CPU type = 68000*/
	sMacOS68020					= 2,							/*Mac OS, CPU type = 68020*/
	sMacOS68030					= 3,							/*Mac OS, CPU type = 68030*/
	sMacOS68040					= 4,							/*Mac OS, CPU type = 68040*/
	board						= 0,							/*Board sResource - Required on all boards*/
	displayVideoAppleTFB		= 16843009,						/*Video with Apple parameters for TFB card.*/
	displayVideoAppleGM			= 16843010,						/*Video with Apple parameters for GM card.*/
	networkEtherNetApple3Com	= 33620225,						/*Ethernet with apple parameters for 3-Comm card.*/
	testSimpleAppleAny			= -2147417856,					/*A simple test sResource.*/
	endOfList					= 255,							/*End of list*/
	defaultTO					= 100							/*100 retries.*/
};

enum {
/* sResource flags for sRsrc_Flags */
	fOpenAtStart				= 1,							/* set => open the driver at start time, else do not */
	f32BitMode					= 2								/* set => a 32-bit address will be put into dctlDevBase (IM Devices 2-54) */
};

enum {
	sRsrcType					= 1,							/*Type of sResource*/
	sRsrcName					= 2,							/*Name of sResource*/
	sRsrcIcon					= 3,							/*Icon*/
	sRsrcDrvrDir				= 4,							/*Driver directory*/
	sRsrcLoadDir				= 5,							/*Load directory*/
	sRsrcBootRec				= 6,							/*sBoot record*/
	sRsrcFlags					= 7,							/*sResource Flags*/
	sRsrcHWDevId				= 8,							/*Hardware Device Id*/
	minorBaseOS					= 10,							/*Offset to base of sResource in minor space.*/
	minorLength					= 11,							/*Length of sResources address space in standard slot space.*/
	majorBaseOS					= 12,							/*Offset to base of sResource in Major space.*/
	majorLength					= 13,							/*Length of sResource in super slot space.*/
	sRsrccicn					= 15,							/*Color icon*/
	sRsrcicl8					= 16,							/*8-bit (indexed) icon*/
	sRsrcicl4					= 17,							/*4-bit (indexed) icon*/
	sDRVRDir					= 16,							/*sDriver directory*/
	sGammaDir					= 64,							/*sGamma directory*/
	sRsrcVidNames				= 65,							/*Video mode name directory*/
	sRsrcDock					= 80,							/*spID for Docking Handlers*/
	sDiagRec					= 85,							/*spID for board diagnostics*/
	sVidAuxParams				= 123,							/*more video info for Display Manager -- timing information*/
	sDebugger					= 124,							/*DatLstEntry for debuggers indicating video anamolies*/
	sVidAttributes				= 125,							/*video attributes data field (optional,word)*/
	fLCDScreen					= 0,							/* bit 0 - when set is LCD, else is CRT*/
	fBuiltInDisplay				= 1,							/*	  1 - when set is built-in (in the box) display, else not*/
	fDefaultColor				= 2,							/*	  2 - when set display prefers multi-bit color, else gray*/
	fActiveBlack				= 3,							/*	  3 - when set black on display must be written, else display is naturally black*/
	fBuiltInDetach				= 4,							/*	  4 - when set is built-in (in the box), but detaches*/
	sVidParmDir					= 126,
	sSuperDir					= 254
};

/* =======================================================================	*/
/* sResource types															*/
/* =======================================================================	*/
enum {
	catBoard					= 1,							/*Category for board types.*/
	catTest						= 2,							/*Category for test types -- not used much.*/
	catDisplay					= 3,							/*Category for display (video) cards.*/
	catNetwork					= 4,							/*Category for Networking cards.*/
	catCPU						= 0x000A,
	catProto					= 0x0011,
	catDock						= 0x0020,
/*<Type>*/
	typeBoard					= 0x0000,
	typeApple					= 0x0001,
	typeVideo					= 0x0001,
	typeEtherNet				= 0x0001,
	typeStation					= 0x0001,
	typeDesk					= 0x0002,
	typeDebugger				= 0x0100,
	type68000					= 0x0002,
	type68020					= 0x0003,
	type68030					= 0x0004,
	type68040					= 0x0005,
	typeAppleII					= 0x0015,
/*Driver Interface : <id.SW>*/
	drSwMacCPU					= 0,
	drSwAppleIIe				= 0x0001,
	drSwApple					= 1,							/*To ask for or define an Apple-compatible SW device.*/
	drSwMacsBug					= 0x0104,
/*Driver Interface : <id.SW><id.HW>*/
	drHwTFB						= 1,							/*HW ID for the TFB (original Mac II) video card.*/
	drHw3Com					= 1,							/*HW ID for the Apple EtherTalk card.*/
	drHwBSC						= 3,
	drHwGemini					= 1,
	drHwDeskBar					= 1,
	drHwBootBug					= 0x0100,
	boardId						= 32,							/*Board Id*/
	pRAMInitData				= 33,							/*sPRAM init data*/
	primaryInit					= 34,							/*Primary init record*/
	timeOutConst				= 35,							/*Time out constant*/
	vendorInfo					= 36,							/*Vendor information List. See Vendor List, below*/
	boardFlags					= 37,							/*Board Flags*/
	secondaryInit				= 38,							/*Secondary init record/code*/
	vendorId					= 1,							/*Vendor Id*/
	serialNum					= 2,							/*Serial number*/
	revLevel					= 3,							/*Revision level*/
	partNum						= 4,							/*Part number*/
	date						= 5								/*Last revision date of the card*/
};

enum {
	testByte					= 32,							/*Test byte.*/
	testWord					= 33,							/*0021*/
	testLong					= 34,							/*Test Long.*/
	testString					= 35							/*Test String.*/
};


#if PRAGMA_IMPORT_SUPPORTED
#pragma import off
#endif

#if STRUCTALIGNMENTSUPPORTED
#pragma options align=reset
#endif

#ifdef __cplusplus
}
#endif

#endif /* __ROMDEFS__ */
