/*
 	File:		StandardSound.h
 
 	Contains:	Public interface to Standard Sound for Cathedral
 
 	Version:	Technology:	
 				Package:	Universal Interfaces  DDK v23c5.  Monday, May 15, 1995.
 
 	Copyright:	 1984-1995 by Apple Computer, Inc.
 				All rights reserved.
 
 	Bugs?:		If you find a problem with this file, use the Apple Bug Reporter
 				stack.  Include the file and version information (from above)
 				in the problem description and send to:
 					Internet:	apple.bugs@applelink.apple.com
 					AppleLink:	APPLE.BUGS
 
*/

#ifndef __STANDARDSOUND__
#define __STANDARDSOUND__


#ifdef __cplusplus
extern "C" {
#endif

#if STRUCTALIGNMENTSUPPORTED
#pragma options align=mac68k
#endif

#if PRAGMA_IMPORT_SUPPORTED
#pragma import on
#endif


#ifndef __DSPMANAGER__
#include <DSPManager.h>
#endif
/*	#include <Types.h>											*/
/*		#include <ConditionalMacros.h>							*/
/*==========================================================================================*/
/*	Patch Bay constants*/
/*==========================================================================================*/

enum PatchPoint {
	kStdSndRecorder,
	kStdSndPreProcessor,
	kStdSndPlayer,
	kStdSndPostProcessor,
	kStdSndInput,
	kStdSndOutput
};

/*==========================================================================================*/
/*	Default Sample Rates (Fixed samples/sec) and Frame Rates (Fixed milliseconds)*/
/*==========================================================================================*/
#define kStdSnd24kHz 0x5DC00000
#define kStdSnd32kHz 0x7D000000
#define kStdSnd48kHz 0xBB800000
#define kStdSnd5ms 0x00050000
#define kStdSnd10ms 0x000A0000
/*==========================================================================================*/
/*	Stream IDs*/
/*==========================================================================================*/
#define kStdSndSystemStream -1
#define kStdSndSystemInputStream -2
#define kStdSndSystemOutputStream -3
/*==========================================================================================*/
/*	Stream Status*/
/*==========================================================================================*/
enum DSPStreamStatus {
	kStdSndStreamIsInactive,
	kStdSndStreamDontKnowIfActive,
	kStdSndStreamIsActive
};

typedef enum DSPStreamStatus DSPStreamStatus;
/*==========================================================================================*/
/*	Structures*/
/*==========================================================================================*/
typedef unsigned long SoundStream;

#ifndef __cplusplus
typedef enum PatchPoint PatchPoint;
typedef struct StdSndStreamInfo StdSndStreamInfo;

#endif
/*==========================================================================================*/
/*	Error Constants*/
/*==========================================================================================*/
#define kStdSndErr -750
#define kNoDSPManagerErr -751
#define kNoDSPDeviceErr -752
#define kOpenDeviceErr -753
#define kStdSndSampleRateNotAvailable -754
#define kStdSndFrameRateNotAvailable -755
#define kStdSndOptionNotAvailable -756
#define kStdSndBadStream -757
#define kStdSndBadIndex -758
#define kStdSndBadInfoVersion -759
/*==========================================================================================*/
/* misc dispatcher constants & macros*/
/*==========================================================================================*/
#define kStdSndGetIndexedStream 128
#define kStdSndGetStreamCPUIndex 129
#define kStdSndInsertTask 130
#define kStdSndGetStreamSections 131
#define kStdSndTestSampleAndFrameRate 132
#define kStdSndSetSampleAndFrameRate 133
#define kStdSndGetStreamInfo 134
#define kStdSndRemoveTask 135
#define kStdSndGetSampleAndFrameRate 136
#define kStdSndSetInputGain 137
#define kStdSndSetOutputAttenuation 138
#define kStdSndGetInputGain 139
#define kStdSndGetOutputAttenuation 140
#define kStdSndGetSampleTime 141
#define kStdSndReserved1 142
#define kStdSndGetMonitorGain 143
#define kStdSndSetMonitorGain 144
#define kStdSndGetInputDeviceConnection 145
#define kStdSndSignInCPUDevice 146
#define kStdSndSignOutCPUDevice 147
#define kStdSndTaskChangingState 148
#define kStdSndGetStreamGivenTask 149
#define kStdSndOnlySystemTasks 150
#define kStdSndGetStreamOutputStatus 151
#define kStdSndGetDeviceInfo 152
#define kStdSndSetDeviceInfo 153
#define kInstallTasks 154
#define kInstallStream 155
#define kDSPLookupDeviceIndex 156
#define kStdSndSetSystemStream 157
#define kStdSndInfoVersion 1
struct StdSndStreamInfo {
	short							stVersion;					/* Version of this info struct*/
	SoundStream						stStream;					/* SoundStream to get info for*/
	PatchPoint						stPatchPoint;				/* Patch point to get info for*/
	unsigned long					stFlags;					/* Type flags*/
	Fixed							stFrameRate;				/* Frame rate in milliseconds*/
	Fixed							stSampleRate;				/* Sample rate in samples/sec*/
	unsigned long					stTime;						/* Time in samples*/
	unsigned short					stCPUDeviceIndex;			/* Device index for this patch point*/
	Str31							stName;						/* Name of this Stream*/
	Handle							stICON;						/* Icon of the Stream*/
};
typedef StdSndStreamInfo *StdSndStreamInfoPtr;

extern pascal OSErr StdSndGetIndexedStream(SoundStream *theStream, long index)
 THREEWORDINLINE(0x303C, 128, 0xABF5);
extern pascal OSErr StdSndGetStreamCPUIndex(SoundStream theStream, PatchPoint patchPoint, unsigned short *cpuDeviceIndex)
 THREEWORDINLINE(0x303C, 129, 0xABF5);
extern pascal OSErr StdSndGetStreamInfo(StdSndStreamInfoPtr theSI)
 THREEWORDINLINE(0x303C, 134, 0xABF5);
extern pascal OSErr StdSndInsertTask(SoundStream theStream, DSPTaskRefNum theTask, PatchPoint patchPoint)
 THREEWORDINLINE(0x303C, 130, 0xABF5);
extern pascal OSErr StdSndRemoveTask(SoundStream theStream, DSPTaskRefNum theTask, PatchPoint patchPoint)
 THREEWORDINLINE(0x303C, 135, 0xABF5);
extern pascal OSErr StdSndGetStreamSections(SoundStream theStream, DSPSectionRefNum *leftChannel, DSPSectionRefNum *rightChannel)
 THREEWORDINLINE(0x303C, 131, 0xABF5);
extern pascal OSErr StdSndTestSampleAndFrameRate(SoundStream theStream, Fixed frame, Fixed sample)
 THREEWORDINLINE(0x303C, 132, 0xABF5);
extern pascal OSErr StdSndSetSampleAndFrameRate(SoundStream theStream, Fixed frame, Fixed sample)
 THREEWORDINLINE(0x303C, 133, 0xABF5);
extern pascal OSErr StdSndGetSampleAndFrameRate(SoundStream theStream, Fixed *frame, Fixed *sample)
 THREEWORDINLINE(0x303C, 136, 0xABF5);
extern pascal OSErr StdSndSetInputGain(SoundStream theStream, unsigned long leftGain, unsigned long rightGain)
 THREEWORDINLINE(0x303C, 137, 0xABF5);
extern pascal OSErr StdSndSetOutputAttenuation(SoundStream theStream, unsigned long leftAttenuation, unsigned long rightAttenuation)
 THREEWORDINLINE(0x303C, 138, 0xABF5);
extern pascal OSErr StdSndGetInputGain(SoundStream theStream, unsigned long *leftGain, unsigned long *rightGain)
 THREEWORDINLINE(0x303C, 139, 0xABF5);
extern pascal OSErr StdSndGetOutputAttenuation(SoundStream theStream, unsigned long *leftAttenuation, unsigned long *rightAttenuation)
 THREEWORDINLINE(0x303C, 140, 0xABF5);
extern pascal OSErr StdSndGetSampleTime(SoundStream theStream, unsigned long *time)
 THREEWORDINLINE(0x303C, 141, 0xABF5);
extern pascal OSErr StdSndGetMonitorGain(SoundStream theStream, Fixed *leftGain, Fixed *rightGain)
 THREEWORDINLINE(0x303C, 143, 0xABF5);
extern pascal OSErr StdSndSetMonitorGain(SoundStream theStream, Fixed leftGain, Fixed rightGain)
 THREEWORDINLINE(0x303C, 144, 0xABF5);
extern pascal OSErr StdSndGetInputDeviceConnection(SoundStream theStream, short *deviceStatus)
 THREEWORDINLINE(0x303C, 145, 0xABF5);
extern pascal OSErr StdSndGetStreamOutputStatus(SoundStream theStream, DSPStreamStatus *streamStatus)
 THREEWORDINLINE(0x303C, 151, 0xABF5);
extern pascal OSErr StdSndGetDeviceInfo(SoundStream theStream, OSType infoType, Ptr infoData)
 THREEWORDINLINE(0x303C, 152, 0xABF5);
extern pascal OSErr StdSndSetDeviceInfo(SoundStream theStream, OSType infoType, Ptr infoData)
 THREEWORDINLINE(0x303C, 153, 0xABF5);

#if PRAGMA_IMPORT_SUPPORTED
#pragma import off
#endif

#if STRUCTALIGNMENTSUPPORTED
#pragma options align=reset
#endif

#ifdef __cplusplus
}
#endif

#endif /* __STANDARDSOUND__ */
