/************************************************************

	ErrNo.h
	Error return codes
	
	Copyright American Telephone & Telegraph
	Used with permission, Apple Computer Inc. 1985-1994
	All rights reserved

************************************************************/

/* Conditional Macros:
 *	UsingStaticLibs	- for CFM-68K:  Insures that #pragma lib_export is never used.
 *	UsingSharedLibs	- for CFM-68K:  Insures that all functions and data items are
 *									marked as exported
 *	<none>			- for CFM-68K:	Data items are exported using #pragma lib_export,
 *									functions are not.  Causes excess code to be
 *									generated for data references to static libraries
 *									and causes the linker to generate glue for
 *									references to shared library routines.
 *	The preceeding macros may not both be defined in the same compilation.
 */

#ifndef __ERRNO__
#define __ERRNO__

#if defined (UsingStaticLibs) && defined (UsingSharedLibs)
	#error "Only one of the conditional macros 'UsingStaticLibs' and 'UsingSharedLibs' may be defined in a compilation"
#endif

#ifdef __CFM68K__
	#ifndef UsingStaticLibs
		#pragma lib_export on
	#endif
#endif

extern int errno;

#ifdef __CFM68K__
	#ifndef UsingSharedLibs
		#pragma lib_export off
	#endif
#endif

#define EPERM		 1 /* Permission denied */
#define ENOENT		 2 /* No such file or directory */
#define ENORSRC 	 3 /* No such resource */
#define EINTR		 4 /* Interrupted system service */
#define EIO 		 5 /* I/O error */
#define ENXIO		 6 /* No such device or address */
#define E2BIG		 7 /* Argument list too long */
#define ENOEXEC	 	 8 /* Exec format error */
#define EBADF		 9 /* Bad file number */
#define ECHILD		10 /* No children processes */
#define EAGAIN		11 /* Resource temporarily unavailable, try again later */
#define ENOMEM		12 /* Not enough space */
#define EACCES		13 /* Permission denied */
#define EFAULT		14 /* Bad address */
#define ENOTBLK 	15 /* Block device required */
#define EBUSY		16 /* Device or resource busy */
#define EEXIST		17 /* File exists */
#define EXDEV		18 /* Cross-device link */
#define ENODEV		19 /* No such device */
#define ENOTDIR 	20 /* Not a directory */
#define EISDIR		21 /* Is a directory */
#define EINVAL		22 /* Invalid argument */
#define ENFILE		23 /* File table overflow */
#define EMFILE		24 /* Too many open files */
#define ENOTTY		25 /* Not a character device */
#define ETXTBSY 	26 /* Text file busy */
#define EFBIG		27 /* File too large */
#define ENOSPC		28 /* No space left on device */
#define ESPIPE		29 /* Illegal seek */
#define EROFS		30 /* Read only file system */
#define EMLINK		31 /* Too many links */
#define EPIPE		32 /* Broken pipe */
#define EDOM		33 /* Math arg out of domain of func */
#define ERANGE		34 /* Math result not representable */

#endif
